/*
 * Originally from x64
 * Modified by Alex Hornby for use in x2600
 * Modified by Daniel Boris for use in Virtual VCS
 * See COPYING for license terms.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include "cpu.h"
#include "macro.h"
#include "vmachine.h"
#include "extern.h"
#include "memory.h"
#include "display.h"
#include "misc.h"
#include "config.h"
#include "exmacro.h"
#include "resource.h"

#include "sega_scl2.h"
#include "sega_per.h"
//#include "sega_int.h"
#include "sega_sys.h"
//#include "sega_pcm.h"
//#include "sega_snd.h"
//#include "../saturn/saturn.h"

	#define	SZ_PERIPHERAL	5

	typedef	Uint8	SysPeripheral[SZ_PERIPHERAL+2];

	typedef	struct	SysPort	{
		Uint8			id;
		Uint8			connectable;
		SysPeripheral	*peripheral;
	} SysPort;

	typedef	struct	SysDevice	{
		Uint8	type;
		Uint8	size;
		Uint8	data[1];
	} SysDevice;

extern volatile SysPort		*__port;
	typedef Uint16	trigger_t;
extern trigger_t	pltrigger[2],pltriggerE[2];
const SysDevice	*PER_GetDeviceR( const SysPort	*port, Uint32	n );
SysPort	*PER_OpenPort( void );
trigger_t	PER_GetTrigger( const SysDevice	*this );
void	PER_GetPort( SysPort	*port );

//#define NO_UNDOC_CMDS 1
//#include "gpdef.h"
//#include "gpstdlib.h"
//#include "gpgraphic.h"

//int verflg=0;
//int hexflg=0;

/* CPU internals */

ADDRESS program_counter;
BYTE    x_register, y_register, stack_pointer;
BYTE    accumulator;
//, status_register;
int 	zero_flag;

int     sign_flag;
int     overflow_flag;
int     break_flag;
int     decimal_flag;
int     interrupt_flag;
int     carry_flag;

CLOCK   clk;
CLOCK clkcount=0;//, clks;
#define NEW2

#define BEAMNINE 5
#define BEAMTWELVE 9
int beamadj;
//extern void test_irq(CLOCK clock);
void    u_stoshr(unsigned int val, ADDRESS address, BYTE index);
extern __inline__ void tv_raster(int line);

static BYTE b;

void init_cpu(ADDRESS addr)
{
  SET_SR(0x20);SP = 0;
  PC = LOAD_ADDR(addr);
  clk=0;
  clkcount=0;
}

void invalid()
{
/*		Set_Old_Int9();
		setmode(TEXT_MODE);
*/

}

void mainloop_init()
{
	/* Initialise the 2600 hardware */
	init_hardware();

	/* Turn the virtual TV on. */
	tv_on();
	SET_SR(0x20);
	SP = 0xFF;
}

static    Uint16 pad_asign[]={
	PER_DGT_U,
	PER_DGT_D,
	PER_DGT_R,
	PER_DGT_L,
	PER_DGT_A,
	PER_DGT_B,
	PER_DGT_C,
	PER_DGT_S,
	PER_DGT_X,
	PER_DGT_Y,
	PER_DGT_TR,
	PER_DGT_TL,
};




/*BYTE mainloop()*/
 void mainloop()
{

	
	while (1) {

	/*keyboard();*/

/*	do_screen(clkcount);*/	/* vmachine.c */
/* do_screen is now inline: */
MCS6510_Exec:

	do_screen(clkcount);
	//clks = clkcount;

MCS6510_Exec2:
/*	  if (app_data.debug) x_loop();*/
	/*b=LOADEXEC(PC);*/
	/* LOADEXEC now inline */
	// VBT a remettre
	if(PC & 0x1000)
		b = theRom[PC & 0xfff];
	else
		b = theRam[PC & 0x7f];
	beamadj=0;
/*	vidbuf.ptbuffer[b] = 0;*/
	unsigned int p1;
	unsigned int p2;
	unsigned short src;
	unsigned short int temp;
	BYTE hb;

switch (b) {
case 0:
	PC+=2;

	SET_BREAK(1);           /* Set break status. */
	PUSH(UPPER(PC));
	PUSH(LOWER(PC));        /* Push return address into the stack. */
	PUSH(GET_SR());                /* Push status register into the stack. */
	SET_INTERRUPT(1);       /* Disable interrupts. */
	PC = LOAD_ADDR((ADDRESS)65534); /* Jump using BRK vector (0xfffe). */
	RET(7);

case 1:
#ifdef NEW2
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
	AC |=  src;
	SET_ZERO(AC);
	SET_SIGN(AC);
	PC+=2;
	RET(6);
#else
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
	PC += 2;

	src |= AC;
	SET_SIGN(src);
	SET_ZERO(src);

	AC = (src);
	RET(6);
#endif

case 2:
case 18:
case 34:
case 50:
case 66:
case 82:
case 98:
case 114:
case 146:
case 178:
case 210:
case 242:
	invalid();
	goto MCS6510_Exec;

case 3:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
	/* src = asl(src); AC = ora(src); */

	SET_CARRY(src & 0x80);	/* ASL+ORA */
	src <<= 1;
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
	AC |= src;
	//src &= 0xff;
	SET_ZERO(AC);
	SET_SIGN(AC);
	PC+=2;
	RET(8);
#else
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
	PC += 2;

	/* src = asl(src); AC = ora(src); */

	SET_CARRY(src & 0x80);	/* ASL+ORA */
	src <<= 1;
	src |= AC;
	src &= 0xff;
	SET_ZERO(src);
	SET_SIGN(src);

	STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
	RET(8);
#endif

case 4:
case 68:
case 100:
	PC += 2;
	RET(3);

case 5:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(LOADEXEC(PC+1));
	AC |= src;
	SET_ZERO(AC);
	SET_SIGN(AC);
	PC+=2;
	RET(3);
#else
	src=LOAD_ZERO(LOADEXEC(PC+1));
	PC += 2;

	src |= AC;
	SET_SIGN(src);
	SET_ZERO(src);

	AC = (src);
	RET(3);
#endif

case 6:
#ifdef NEW2
	p1 = LOAD(PC+1);
	src=LOAD_ZERO(p1);
	SET_CARRY(src & 0x80);
	src <<= 1;
	//src &= 0xff;
	SET_ZERO(src);
	SET_SIGN(src);

	STORE_ZERO(p1, (src));
	PC+=2;
	RET(5);
#else
	p1 = LOAD(PC+1);
	src=LOAD_ZERO(p1);
	PC += 2;
	SET_CARRY(src & 0x80);
	src <<= 1;
	src &= 0xff;
	SET_SIGN(src);
	SET_ZERO(src);

	STORE_ZERO(p1, (src));
	RET(5);
#endif


case 7:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1);
	SET_CARRY(src & 0x80);	/* ASL+ORA */
	src <<= 1;
	AC |= src ;
	//src &= 0xff;
	SET_ZERO(AC);
	SET_SIGN(AC);
	STORE_ZERO(p1, (src));
    PC +=2;
	RET(5);
#else
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1);
    PC += 2;
	SET_CARRY(src & 0x80);	/* ASL+ORA */
	src <<= 1;
	src |= AC;
	src &= 0xff;
	SET_SIGN(src);
	SET_ZERO(src);
    
    STORE_ZERO(p1, (src));
	RET(5);
#endif


case 8:
    src=GET_SR();
	PC++;

	PUSH(src);
	/* First push item onto stack and then decrement SP. */
	RET(3);

case 9:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
	AC |= src;
	SET_ZERO(AC);
	SET_SIGN(AC);
	PC += 2;
	RET(2);
#else
 	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
	PC += 2;

	src |= AC;
	SET_SIGN(src);
	SET_ZERO(src);

	AC = (src);
	RET(2);
#endif

case 10:
#ifdef NEW2
    SET_CARRY(AC & 0x80);
	AC <<= 1;
    SET_ZERO(AC);
	SET_SIGN(AC);
    PC++;
	RET(2);
#else
	src=AC;
    PC++;

    SET_CARRY(src & 0x80);
	src <<= 1;
    src &= 0xff;
	SET_SIGN(src);
    SET_ZERO(src);

	AC = (src);
	RET(2);
#endif

case 11:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
    src=(AC & LOADEXEC(PC+1));
    AC &= src;
    SET_ZERO(AC);
	SET_SIGN(AC);
	PC += 2;
	RET(2);
#else
	//p1 = LOADEXEC(PC+1);
    src=(AC & LOADEXEC(PC+1));
	PC += 2;

    src &= AC;
	SET_SIGN(src);
    SET_ZERO(src);
    
	AC = (src);
	RET(2);
#endif

case 12:
case 28:
case 60:
case 92:
case 124:
case 220:
case 252:
    PC += 3;
	RET(4);

case 13:
#ifdef NEW2
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);//src=LOAD(p2);
	AC |= src;
	SET_ZERO(AC);
	SET_SIGN(AC);
	PC += 3;
	RET(4);
#else
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);//src=LOAD(p2);
	PC += 3;

	src |= AC;
	SET_SIGN(src);
	SET_ZERO(src);

	AC = (src);
	RET(4);
#endif

case 14:
#ifdef NEW2
	p2 = load_abs_addr;
    src=LOAD(p2);
    SET_CARRY(src & 0x80);
	src <<= 1;
    //src &= 0xff;
    STORE(p2, (src));
    SET_ZERO(src);
	SET_SIGN(src);
    PC += 3;
	RET(6);
#else
	p2 = load_abs_addr;
    src=LOAD(p2);
    PC += 3;

    SET_CARRY(src & 0x80);
	src <<= 1;
    src &= 0xff;
	SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));
	RET(6);
#endif

case 15:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2);
	SET_CARRY(src & 0x80);	/* ASL+ORA */
	src <<= 1;
    STORE(p2, (src));
	AC |= src;
	//AC &= 0xff;
    SET_ZERO(AC);
	SET_SIGN(AC);
	PC += 3;
	RET(6);
#else
	p2 = load_abs_addr;
	src=LOAD(p2);
	PC += 3;

	/* src = asl(src); AC = ora(src); */

	SET_CARRY(src & 0x80);	/* ASL+ORA */
	src <<= 1;
	src |= AC;
	src &= 0xff;
	SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));
	RET(6);
#endif

case 16:
/*  BPL,    RELATIVE */
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
		PC += 2;	
	/* Branch if sign flag is clear. */
	if (!IF_SIGN())	{
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
			/* Same page */
			RET(3);
		}
		else
		{
			/* Different page */
			RET(4);
		}
	}
	else
	{
		RET(2);
	}

case 17:
#ifdef NEW2
/* ORA,    INDIRECT_Y */
	//p1 = LOADEXEC(PC+1);
	p2 = LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);

    AC |= src;
    SET_ZERO(AC);
    SET_SIGN(AC);

	PC += 2;
	/* If low byte of address  + index reg is > 0xff then extra cycle */
	if( pagetest(p2,YR))
	{
	  /* Over page */
       RET(6);
	}
	else
	{
      /* Same page */
      RET(5);
	}
#else
/* ORA,    INDIRECT_Y */
	//p1 = LOADEXEC(PC+1);
	p2 = LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
	PC += 2;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);

	/* If low byte of address  + index reg is > 0xff then extra cycle */
	if( pagetest(p2,YR))
	{
	  /* Over page */
       RET(6);
	}
	else
	{
      /* Same page */
      RET(5);
	}
#endif

case 19:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    SET_CARRY(src & 0x80);	/* ASL+ORA*/
    src <<= 1;
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));
    AC |= src;
    AC &= 0xff;
    SET_ZERO(AC);
	SET_SIGN(AC);
    PC += 2;
	RET(8);
#else
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    PC += 2;

    /* src = asl(src); AC = ora(src); */

    SET_CARRY(src & 0x80);	/* ASL+ORA*/
    src <<= 1;
    src |= AC;
    src &= 0xff;
	SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));;    
	RET(8);
#endif

case 20:
case 52:
case 84:
case 116:
case 212:
case 244:
    PC += 2;
	RET(4);

case 21:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
	AC |= src;
	SET_ZERO(AC);
	SET_SIGN(AC);
    PC += 2;
	RET(4);
#else
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    PC += 2;

	src |= AC;
	SET_SIGN(src);
	SET_ZERO(src);

	AC = (src);
	RET(4);
#endif

case 22:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    SET_CARRY(src & 0x80);
	src <<= 1;
    STORE_ZERO(p1 + XR, (src));
	//src &= 0xff;
	SET_ZERO(src);
	SET_SIGN(src);
	PC += 2;
	RET(6);
#else
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
	PC += 2;

    SET_CARRY(src & 0x80);
	src <<= 1;
	src &= 0xff;
	SET_SIGN(src);
	SET_ZERO(src);

    STORE_ZERO(p1 + XR, (src));
	RET(6);
#endif

case 23:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1 + XR);
    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    STORE_ZERO(p1 + XR, (src));
    AC |= src;
    //AC &= 0xff;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
	RET(6);
#else
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1 + XR);
    PC += 2;

    /* src = asl(src); AC = ora(src); */

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
	SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));
	RET(6);
#endif

case 24:
#ifdef NEW2
   SET_CARRY((0));
    PC++;
   RET(2);
#else
    PC++;
   SET_CARRY((0));
   RET(2);
#endif


case 25:
#ifdef NEW2
/*  ORA,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    AC |= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
    if( pagetest(p2,YR))
	{
      /* Over page */
	  RET(5);
	}
    else
	{
      /* Same page */
	  RET(4);
	}
#else
/*  ORA,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;
    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
    if( pagetest(p2,YR))
	{
      /* Over page */
	  RET(5);
	}
    else
	{
      /* Same page */
	  RET(4);
	}
#endif

case 26:
case 58:
case 90:
case 122:
case 218:
case 234:
case 250:
    PC++;
	RET(2);

case 27:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    STORE(p2 + YR, (src));
    AC |= src;
    //AC &= 0xff;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
	RET(7);
#else
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;

    /* src = asl(src); AC = ora(src); */

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + YR, (src));
	RET(7);
#endif

case 29:
#ifdef NEW2
/*   ORA,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    AC |= AC;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
    if( pagetest(p2,XR))
	{
      /* Over page */
		RET(5);
	}
    else
	{
      /* Same page */
		RET(4);
	}
#else
/*   ORA,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    src |= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);

    if( pagetest(p2,XR))
	{
      /* Over page */
		RET(5);
	}
    else
	{
      /* Same page */
		RET(4);
	}
#endif

case 30:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    SET_CARRY(src & 0x80);
    src <<= 1;
    STORE(p2 + XR, (src));
    //src &= 0xff;
    SET_ZERO(src);
    SET_SIGN(src);
    PC += 3;
	RET(7);
#else
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    SET_CARRY(src & 0x80);
    src <<= 1;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));
	RET(7);
#endif

case 31:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    STORE(p2 + XR, (src));
    AC |= src;
    //src &= 0xff;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
	RET(7);
#else
 	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    /* src = asl(src); AC = ora(src); */

    SET_CARRY(src & 0x80);	/* ASL+ORA */
    src <<= 1;
    src |= AC;
    src &= 0xff;
	SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));
	RET(7);
#endif
case 32:
#ifdef NEW2
	//p2 = load_abs_addr;
    src=load_abs_addr;
    PC += 2;
		   /* Jump to subroutine. */
	//PC--;
	PUSH((PC >> 8) & 0xff);	/* Push return address onto the stack. */
	PUSH(PC & 0xff);

	PC = (src);
	RET(6);
#else
	//p2 = load_abs_addr;
    src=load_abs_addr;
    PC += 3;
		   /* Jump to subroutine. */
	PC--;
	PUSH((PC >> 8) & 0xff);	/* Push return address onto the stack. */
	PUSH(PC & 0xff);

	PC = (src);
	RET(6);
#endif

case 33:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
    AC &= src;
	SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
	RET(6);
#else
 	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
    PC += 2;

    src &= AC;
    SET_SIGN(src);
	SET_ZERO(src);
    
    AC = (src);
	RET(6);
#endif

case 35:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
	src <<= 1;
	if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
    AC &= src;
    SET_CARRY(src & 0x80);
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
	RET(8);
#else
 	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
    PC += 2;

	/* src = rol(src); AC = and(src); */

	src <<= 1;
	if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
	RET(8);
#endif

case 36:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    SET_ZERO(AC & src);
    SET_SIGN(src);
    SET_OVERFLOW(src & 0x40);	/* Copy bit 6 to OVERFLOW flag. */
    PC += 2;
	RET(3);
#else
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    PC += 2;

    SET_SIGN(src);
    SET_OVERFLOW(0x40 & src);	/* Copy bit 6 to OVERFLOW flag. */
    SET_ZERO(src & AC);
	RET(3);
#endif

case 37:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(LOADEXEC(PC+1));
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
	PC += 2;
	RET(3);
#else
	//p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(LOADEXEC(PC+1));
	PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(3);
#endif

case 38:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1);
	p2 = IF_CARRY();

    SET_CARRY(src & 0x80);
    src <<= 1;
    if (p2)
		src |= 0x1;
    //src &= 0xff;
    STORE_ZERO(p1, (src));
    SET_ZERO(src);
    SET_SIGN(src);
    PC += 2;
	RET(5);
#else
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1);
    PC += 2;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));
	RET(5);
#endif

case 39:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1);
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    STORE_ZERO(p1, (src));
    AC &= src;
    SET_CARRY(src & 0x80);
    //src &= 0xff;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
	RET(5);
#else
 	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1);
    PC += 2;

    /* src = rol(src); AC = and(src); */

    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE_ZERO(p1, (src));
	RET(5);
#endif

case 40:
	PC++;

    /* First increment stack pointer and then pull item from stack. */
    src = PULL();
    
    SET_SR((src));
	RET(4);

case 41:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
	RET(2);
#else
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);
#endif

case 42:
#ifdef NEW2
	p2=CF;
    SET_CARRY(AC & 0x80);
    AC <<= 1;
    if (p2)
	AC |= 0x1;
    //AC &= 0xff;
    SET_ZERO(AC);
    SET_SIGN(AC);
	PC++;	;
	RET(2);
#else
	src=AC;
	PC++;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	RET(2);
#endif
case 43:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
	src=(AC & LOADEXEC(PC+1));
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
	RET(2);
#else
	//p1 = LOADEXEC(PC+1);
	src=(AC & LOADEXEC(PC+1));
    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);
#endif

case 44:
#ifdef NEW2
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    SET_ZERO(src & AC);
    SET_SIGN(src);
    SET_OVERFLOW(0x40 & src);	/* Copy bit 6 to OVERFLOW flag. */
    PC += 3;
	RET(4);
#else
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

    SET_SIGN(src);
    SET_OVERFLOW(0x40 & src);	/* Copy bit 6 to OVERFLOW flag. */
    SET_ZERO(src & AC);
	RET(4);
#endif

case 45:
#ifdef NEW2
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
	RET(4);
#else
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(4);
#endif

case 46:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2);
	p1 = CF;
    SET_CARRY(src & 0x80);
    src <<= 1;
    if (p1)
		src |= 0x1;
    STORE(p2, (src));
    //src &= 0xff;
    SET_ZERO(src);
    SET_SIGN(src);
   
    PC += 3;
	RET(6);
#else
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));
	RET(6);
 #endif
case 47:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2);
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
		src |= 0x1;
    STORE(p2, (src));
    AC &= src;
    SET_CARRY(src & 0x80);
    //src &= 0xff;
     SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
	RET(6);
#else
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    /* src = rol(src); AC = and(src); */

    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(p2, (src));
	RET(6);
#endif
case 48:
#ifdef NEW2
/*   BMI,    RELATIVE */
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
	    PC += 2;
	       /* Branch if sign flag is set. */
	if (IF_SIGN())
	{
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		  /* Same page */
		  RET(3);
		}
		else
		{
		  /* Different page */
		  RET(4);
		}
	}
    else
	{
		RET(2);
	}
#else
 /*   BMI,    RELATIVE */
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
    PC += 2;
	       /* Branch if sign flag is set. */
	if (IF_SIGN())
	{
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		  /* Same page */
		  RET(3);
		}
		else
		{
		  /* Different page */
		  RET(4);
		}
	}
    else
	{
		RET(2);
	}
#endif

case 49:
#ifdef NEW2
    p2 = LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
    if( pagetest(p2,YR))
	{
      /* Over page */
       RET(6);
	}
    else
	{
      /* Same page */
		RET(5);
	}
#else
/*  AND,    INDIRECT_Y*/
	//p1 = LOADEXEC(PC+1);
    p2 = LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);

    if( pagetest(p2,YR))
	{
      /* Over page */
       RET(6);
	}
    else
	{
      /* Same page */
		RET(5);
	}
#endif

case 51:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));
    AC &= src;
    SET_CARRY(AC & 0x80);
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
    RET(8);
#else
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    PC += 2;

    /* src = rol(src); AC = and(src); */

    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));
	RET(8);
#endif

case 53:
#ifdef NEW2
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 2;
	RET(4);
#else
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    PC += 2;

    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(4);
#endif

case 54:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
	p2 = CF;
    SET_CARRY(src & 0x80);
    src <<= 1;
    if (p2)
		src |= 0x1;
    STORE_ZERO(p1 + XR, (src));
//    src &= 0xff;
    SET_ZERO(src);
    SET_SIGN(src);
    PC += 2;
	RET(6);
 #else
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    PC += 2;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));
	RET(6);
#endif
case 55:
#ifdef NEW2
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
    src |= 0x1;
    STORE_ZERO(p1 + XR, (src));
    AC &= src;
    SET_CARRY(src & 0x80);
    SET_ZERO(AC);
    //src &= 0xff;
    SET_SIGN(AC);
    PC += 2;
	RET(6);
#else
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    PC += 2;

    /* src = rol(src); AC = and(src); */

    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE_ZERO(p1 + XR, (src));
	RET(6);
#endif
case 56:
    SET_CARRY((1));
    PC++;
	RET(2);

case 57:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
	if( pagetest(p2,YR))
	{
  /* Over page */
		RET(5);
	}
	else
	{
  /* Same page */
		RET(4);
	}
#else
/*  AND,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;
    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	if( pagetest(p2,YR))
	{
  /* Over page */
		RET(5);
	}
	else
	{
  /* Same page */
		RET(4);
	}
#endif

case 59:
#ifdef NEW2
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
    src |= 0x1;
    STORE(p2 + YR, (src));
    AC &= src;
    SET_CARRY(src & 0x80);
    //src &= 0xff;
     SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
    RET(7);
#else
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;

    /* src = rol(src); AC = and(src); */

    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(p2 + YR, (src));
	RET(7);
#endif

case 61:
#ifdef NEW2
/*   AND,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    AC &= src;
    SET_ZERO(AC);
    SET_SIGN(AC);
    PC += 3;
    if( pagetest(p2,XR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}
#else
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    src &= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
    PC += 3;
    if( pagetest(p2,XR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}
#endif

case 62:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    src <<= 1;
    if (IF_CARRY())
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));
	RET(7);

case 63:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    /* src = rol(src); AC = and(src); */

    src <<= 1;
    if (IF_CARRY())	/* ROL+AND */
	src |= 0x1;
    SET_CARRY(src > 0xff);
    src &= 0xff;

    AC &= src;
    SET_SIGN(AC);
    SET_ZERO(AC);
    	/* rotated, not disturbed */
    STORE(p2 + XR, (src));
	RET(7);

case 64:
    PC++;
	       /* Return from interrupt. */
    /* Load program status and program counter from stack. */
    src = PULL();
    SET_SR(src);
    src = PULL();
    src |= (PULL() << 8);	/* Load return address from stack. */

#ifdef DEBUG
    if (traceflg)
	print_stack(SP);
#endif
    PC = (src);
	RET(6);

case 65:
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);
	RET(6);

case 67:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
    PC += 2;

    /* src = lsr(src); AC = eor(src); */

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src ^= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
	RET(8);

case 69:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);
	RET(3);

case 70:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1);
    PC += 2;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));
	RET(5);

case 71:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1);
    PC += 2;

    /* src = lsr(src); AC = eor(src); */

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src ^= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    STORE_ZERO(p1, (src));
	RET(5);

case 72:
/*   PHA,    IMPLIED */
//    src=AC;
    PC++;

    beamadj=BEAMNINE;
    PUSH(AC);
    /* First push item onto stack and then decrement SP. */
    RET(3);

case 73:
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
    PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);
	RET(2);

case 74:
	src=AC;
	PC++;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);

case 75:
	//p1 = LOADEXEC(PC+1);
	src=(AC & LOADEXEC(PC+1));
    PC += 2;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);

case 76:
	//p2 = load_abs_addr;
	//src=load_abs_addr;
    //PC += 3;
	//PC = (src);
	PC = (load_abs_addr);
	RET(3);

case 77:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
	PC += 3;

	src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);
	RET(4);

case 78:
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));
	RET(6);

case 79:
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    /* src = lsr(src); AC = eor(src); */

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src ^= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    STORE(p2, (src));
	RET(6);

case 80:
/*   BVC,    RELATIVE */
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
	PC += 2;
	       /* Branch if overflow flag is clear. */
    if (!IF_OVERFLOW())
    {
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		  /* Same page */
			RET(3);
		}
		else
		  /* Different page */
			RET(4);
    }
	else
	{
		RET(2);
	}

case 81:
/* EOR,    INDIRECT_Y */
	//p1 = LOADEXEC(PC+1);
    p2=LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    PC += 2;
    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);

    if( pagetest(p2,YR))
	{
      /* Over page */
		RET(6);
	}
	else
	{
      /* Same page */
		RET(5);
	}

case 83:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    PC += 2;

    /* src = lsr(src); AC = eor(src); */

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src ^= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));
	RET(8);

case 85:
	//p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
	PC += 2;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);
	RET(4);

case 86:
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1 + XR);
	PC += 2;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));
	RET(6);

case 87:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    PC += 2;

    /* src = lsr(src); AC = eor(src); */

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src ^= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
	STORE_ZERO(p1 + XR, (src));
	RET(6);

case 88:
	PC++;
	SET_INTERRUPT((0));
	RET(2);

case 89:
/*  EOR,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;

    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);

    if( pagetest(p2,YR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
		RET(4);
	}

case 91:
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;

    /* src = lsr(src); AC = eor(src); */

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src ^= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);

    STORE(p2 + YR, (src));
	RET(7);

case 93:
/*  EOR,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;
    src ^= AC;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src & 0xff);
    if( pagetest(p2,XR))
	{
      /* Over page */
		RET(5);
	}
    else
	{
      /* Same page */
		RET(4);
	}

case 94:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));
	RET(7);

case 95:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    /* src = lsr(src); AC = eor(src); */

    SET_CARRY(src & 0x01);	/* LSR+EOR */
    src >>= 1;
    src ^= AC;
    src &= 0xff;
    SET_SIGN(src);
    SET_ZERO(src);

    
    STORE(p2 + XR, (src));
	RET(7);

case 96:
    PC++;
	       /* Return from subroutine. */
    src = PULL();
    src += ((PULL()) << 8) + 1;	/* Load return address from stack and add 1. */

#ifdef DEBUG
    if (traceflg)
	print_stack(SP);
#endif

    
    PC = (src);
	RET(6);

case 97:
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
    temp = src + AC + (IF_CARRY() ? 1 : 0);    PC += 2;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);
	RET(6);

case 99:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
	PC += 2;


    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(LOAD_ZERO_ADDR(p1 + XR), ((BYTE) temp));
	RET(8);

case 101:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    temp = src + AC + (IF_CARRY() ? 1 : 0);    PC += 2;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);
	RET(3);

case 102:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1);
    PC += 2;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1, (src));
	RET(5);

case 103:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
	PC += 2;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    STORE_ZERO(p1, ((BYTE) temp));
	RET(5);

case 104:
/*   PLA,    IMPLIED */
    PC++;

    /* First increment stack pointer and then pull item from stack. */
    src = PULL();
    SET_SIGN(src);	/* Change sign and zero flag accordingly. */
    SET_ZERO(src);
    
    AC = (src);
	RET(4);

case 105:
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    temp = src + AC + (IF_CARRY() ? 1 : 0);    PC += 2;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);
	RET(2);

case 106:
    src=AC;
    PC++;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);

case 107:
	//p1 = LOADEXEC(PC+1);
    src=(AC & LOADEXEC(PC+1));
    PC += 2;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);

case 108:
	//p2 = load_abs_addr;
    //src=LOAD_ADDR(load_abs_addr);
    //PC += 3;
    
    //PC = (src);
	PC = (LOAD_ADDR(load_abs_addr));
	RET(5);

case 109:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    temp = src + AC + (IF_CARRY() ? 1 : 0);    PC += 3;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);
	RET(4);

case 110:
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));
	RET(6);

case 111:
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }
    
    STORE(p2, ((BYTE) temp));
	RET(6);

case 112:
/*   BVS,    RELATIVE */
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;
	       /* Branch if overflow flag is set. */
    if (IF_OVERFLOW())
    {
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		  /* Same page */
			RET(3);
		}
		else
		{
		  /* Different page */
		  RET(4);
		}  
	}
    else
	{
        RET(2);
	}

case 113:
	//p1 = LOADEXEC(PC+1);
    p2=LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    temp = src + AC + (IF_CARRY() ? 1 : 0);    
	PC += 2;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);

	if( pagetest(p2,YR))
	{
      /* Over page */
		RET(6);
	}
    else
	{
      /* Same page */
      RET(5);
	}

case 115:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    PC += 2;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(LOAD_ZERO_ADDR(p1) + YR, ((BYTE) temp));
	RET(8);

case 117:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    temp = src + AC + (IF_CARRY() ? 1 : 0);    PC += 2;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);
	RET(4);

case 118:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    PC += 2;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));
	RET(6);

case 119:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    PC += 2;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }
    
    STORE_ZERO(p1 + XR, ((BYTE) temp));
	RET(6);

case 120:
    PC++;
    SET_INTERRUPT((1));
	RET(2);

case 121:
/*   ADC,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    temp = src + AC + (IF_CARRY() ? 1 : 0);    PC += 3;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);
	if( pagetest(p2,YR))
	{
      /* Over page */
       RET(5);
	}
    else
	{
      /* Same page */
		RET(4);
	}

case 123:
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;
    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(p2 + YR, ((BYTE) temp));
	RET(7);

case 125: 
/*  ADC,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    temp = src + AC + (IF_CARRY() ? 1 : 0);    PC += 3;

    SET_ZERO(temp & 0xff);	/* This is not valid in decimal mode */

    /*
     * In decimal mode this is "correct" but not exact emulation. However,
     * the probability of different result when using undefined BCD codes is
     * less than 6%
     * Sign and Overflow are set between BCD fixup of the two digits  -MSM
     */

    if (IF_DECIMAL()) {
        if (((AC & 0xf) + (src & 0xf) + (IF_CARRY() ? 1 : 0)) > 9)
            temp += 6;

	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));

	if (temp > 0x99)
	    temp += 96;

	SET_CARRY(temp > 0x99);
    } else {
	SET_SIGN(temp);
	SET_OVERFLOW(!((AC ^ src) & 0x80) && ((AC ^ temp) & 0x80));
	SET_CARRY(temp > 0xff);
    }
    
    AC = ((BYTE) temp);

	if( pagetest(p2,XR))
	{
      /* Over page */
       RET(5);
	}
    else
	{
      /* Same page */
		RET(4);
	}

case 126:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    if (IF_CARRY())
	src |= 0x100;
    SET_CARRY(src & 0x01);
    src >>= 1;

    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));
	RET(7);

case 127:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    /* src = ror(src); AC = adc(src);
     * the real operation discovered by msmakela
     * ADC only does the BCD fixup */

    temp = src >>1;	/* ROR+ADC */
    if (IF_CARRY())
	temp |= 0x80;

    SET_SIGN(temp);
    SET_ZERO(temp);			/* Set flags like ROR does */

    SET_OVERFLOW((temp ^ src) & 0x40);	/* ROR causes this ? */

    if (IF_DECIMAL()) {
        if ((src & 0x0f) > 4)		/* half of the normal */
            temp = (temp & 0xf0) | ((temp + 6) & 0xf);
        if (src > 0x4f)
            temp += 96;

	SET_CARRY(src > 0x4f);
    } else {
	SET_CARRY(src & 0x80);		/* 8502 behaviour */
    }

    
    STORE(p2 + XR, ((BYTE) temp));
	RET(7);

case 128:
case 130:
case 137:
case 194:
case 226:
    PC += 2;
	RET(2);

case 129:
	p1 = LOADEXEC(PC+1);
    src=AC;
    PC += 2;
    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
	RET(6);

case 131:
	p1 = LOADEXEC(PC+1);
    src=(AC & XR);
    PC += 2;
    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
	RET(6);

case 132:
/*  STY,    ZERO_PAGE */
	p1 = LOADEXEC(PC+1);
    src=YR;
    PC += 2;

    beamadj=BEAMNINE;    
    STORE_ZERO(p1, (src));
	RET(3);

 case 133:
/*   STA,    ZERO_PAGE */
	p1 = LOADEXEC(PC+1);
    src=AC;
    PC += 2;

    beamadj=BEAMNINE;    
    STORE_ZERO(p1, (src));

	/*vidbuf.ptbuffer[0] = 0;*/
	RET(3);

case 134:
/*  STX,    ZERO_PAGE */
	p1 = LOADEXEC(PC+1);
    src=XR;
    PC += 2;

    beamadj=BEAMNINE;    
	STORE_ZERO(p1, (src));
	RET(3);

case 135:
	p1 = LOADEXEC(PC+1);
    src=(AC & XR);
    PC += 2;
   
    STORE_ZERO(p1, (src));
	RET(3);

case 136:
	PC++;
	YR--;
	SET_SIGN(YR);
	SET_ZERO(YR);
	RET(2);

case 138:
    src=XR;
    PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);

case 139:
	//p1 = LOADEXEC(PC+1);
    src=((AC | 0xee) & XR & LOADEXEC(PC+1));
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(2);

case 140:
/*  STY,    ABSOLUTE */
	p2 = load_abs_addr;
    src=YR;
    PC += 3;

    beamadj=BEAMTWELVE;
    STORE(p2, (src));
	RET(4);

case 141:
/*  STA,    ABSOLUTE */
	p2 = load_abs_addr;
    src=AC;
    PC += 3;

    beamadj=BEAMTWELVE;
    STORE(p2, (src));
	RET(4);

case 142:
/*  STA,    ABSOLUTE */
	p2 = load_abs_addr;
    src=XR;
    PC += 3;

    beamadj=BEAMTWELVE;
    STORE(p2, (src));
	RET(4);

case 143:
/* STX,    ABSOLUTE */
	p2 = load_abs_addr;
    src=(AC & XR);
    PC += 3;

    beamadj=BEAMTWELVE;
    STORE(p2, (src));
	RET(4);

case 144:
/*  BCC,    RELATIVE */
//	p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;
	       /* Branch if carry flag is clear. */
    if (!IF_CARRY())
    {
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		  /* Same page */
		  RET(3);
		}
		else
		{
		  /* Different page */
		  RET(4);
		}
    }
    else 
	{
      RET(2);
	}

case 145:
	p1 = LOADEXEC(PC+1);
    src=AC;
    PC += 2;
    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));
	RET(6);

case 147:
#ifndef NO_UNDOC_CMDS
	p1 = LOADEXEC(PC+1);
#endif
    src=(AC & XR);
    PC += 2;

#ifndef NO_UNDOC_CMDS    
    u_stoshr(src, LOAD_ZERO_ADDR(p1), YR);
#endif
	RET(6);

case 148:
	p1 = LOADEXEC(PC+1);
    src=YR;
    PC += 2;
    
    STORE_ZERO(p1 + XR, (src));
	RET(4);

case 149:
/*  STA,    ZERO_PAGE_X */ 
	p1 = LOADEXEC(PC+1);
    src=AC;
    PC += 2;

    beamadj=BEAMTWELVE;
    STORE_ZERO(p1 + XR, (src));
	RET(4);

case 150:
	p1 = LOADEXEC(PC+1);
    src=XR;
    PC += 2;
    
    STORE_ZERO(p1 + YR, (src));
	RET(4);

case 151:
	p1 = LOADEXEC(PC+1);
    src=(AC & XR);
    PC += 2;
    
    STORE_ZERO(p1 + YR, (src));
	RET(4);

case 152:
    src=YR;
	PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);;
	RET(2);

case 153:
	p2 = load_abs_addr;
	src=AC;
    PC += 3;
    
    STORE(p2 + YR, (src));
	RET(5);

case 154:
    src=XR;
    PC++;
    
    SP = (src);
	RET(2);

case 155:
    src=(AC & XR);
    PC += 3;
	SP = src;	/* SHS */;
	RET(5);

case 156:
#ifndef NO_UNDOC_CMDS 
	p2 = load_abs_addr;
#endif
    src=YR;
    PC += 3;

#ifndef NO_UNDOC_CMDS    
    u_stoshr(src, p2, XR);
#endif
	RET(5);

case 157:
	p2 = load_abs_addr;
    src=AC;
    PC += 3;
    
    STORE(p2 + XR, (src));
	RET(5);

case 158:
#ifndef NO_UNDOC_CMDS 
	p2 = load_abs_addr;
#endif
    src=XR;
    PC += 3;

#ifndef NO_UNDOC_CMDS    
    u_stoshr(src, p2, YR);
#endif
	RET(5);

case 159:
#ifndef NO_UNDOC_CMDS
	p2 = load_abs_addr;
#endif
    src=(AC & XR);
    PC += 3;

#ifndef NO_UNDOC_CMDS    
    u_stoshr(src, p2, YR);
#endif
	RET(5);

case 160:
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);
	RET(2);

case 161:
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(6);

case 162:
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);
	RET(2);

case 163:
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
	AC = XR = (src);
	RET(6);

case 164:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);

    YR = (src);
	RET(3);

case 165:
	//p1 = LOADEXEC(PC+1);

	AC=LOAD_ZERO(LOADEXEC(PC+1));
	PC += 2;

	SET_SIGN(AC);
	SET_ZERO(AC);

	RET(3);

case 166:
/*	LDX,   ZERO_PAGE*/
	//p1 = LOADEXEC(PC+1);
	XR=LOAD_ZERO(LOADEXEC(PC+1));

	PC += 2;

	SET_SIGN(XR);
	SET_ZERO(XR);
	RET(3);

case 167:
	//p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(LOADEXEC(PC+1));
	PC += 2;

	SET_SIGN(src);
	SET_ZERO(src);

	AC = XR = (src);
	RET(3);

case 168:
	PC++;

	SET_SIGN(AC);
	SET_ZERO(AC);

	YR = AC;
	RET(2);

case 169:
	AC=LOADEXEC(PC+1);
	PC+=2;
	SET_SIGN(AC);
	SET_ZERO(AC);
	RET(2);

case 170:
	PC++;

	SET_SIGN(AC);
	SET_ZERO(AC);

	XR = AC;
	RET(2);

case 171:
	//p1 = LOADEXEC(PC+1);
    src=(AC & LOADEXEC(PC+1));
    PC += 2;

	SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);
	RET(2);

case 172:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);
	RET(4);

case 173: /* LDA absolute */
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(4);
	/*GpTextOut(NULL, &vidbuf.ptbuffer, 0, 0, "prova", 1);*/

case 174:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);
	RET(4);

case 175:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);
	RET(4);

case 176:
/*   BCS,    RELATIVE */
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;
	       /* Branch if carry flag is set. */
    if (IF_CARRY())
    {
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		  /* Same page */
			RET(3);
		}
		else
		{
		  /* Different page */
			RET(4);
		}
    }
    else
	{
		RET(2);
	}

case 177:
/*   LDA,    INDIRECT_Y */
	//p1 = LOADEXEC(PC+1);
    p2 = LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
    
    if( pagetest(p2,YR))
	{
      /* Over page */
       RET(6);
	}
    else
	{
      /* Same page */
	  RET(5);
	}

case 179:
/*   LAX,    INDIRECT_Y */
	//p1 = LOADEXEC(PC+1);
    p2=LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    PC += 2;
    
    SET_SIGN(src);
    SET_ZERO(src);

    AC = XR = (src);

    if( pagetest(p2,YR))
	{
      /* Over page */
       RET(6);
	}
    else
	{
      /* Same page */
      RET(5);
	}

case 180:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    PC += 2;

    SET_SIGN(src);
	SET_ZERO(src);
    
    YR = (src);
	RET(4);

case 181:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    PC += 2;

	SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);
	RET(4);

case 182:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + YR);
    PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);
	RET(4);

case 183:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + YR);
	PC += 2;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);
	RET(4);

case 184:
    PC++;
    SET_OVERFLOW((0));
	RET(2);

case 185:
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);

    if( pagetest(p2,YR))
	{
	  /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4); 
	}

case 186:
    src=SP;
    PC++;

    SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);
	RET(2);

case 187:
	p2 = load_abs_addr;
	src=(SP & LOAD(p2 + YR));
    PC += 3;
    
	SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = SP = (src);
    if( pagetest(p2,YR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}

case 188:
/*   LDY,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
	PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    YR = (src);

    if( pagetest(p2,XR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}

case 189: 
/*  LDA,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;
    
    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = (src);

    if( pagetest(p2,XR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}

case 190:
/*   LDX,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;

	SET_SIGN(src);
    SET_ZERO(src);
    
    XR = (src);

    if( pagetest(p2,YR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}

case 191:
/*   LAX,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
	PC += 3;

    SET_SIGN(src);
    SET_ZERO(src);
    
    AC = XR = (src);

    if( pagetest(p2,YR))
	{
      /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}

case 192:
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;

    src = YR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
	RET(2);

case 193:
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
	PC += 2;

    src = AC - src;
    SET_CARRY(src < 0x100);
	SET_SIGN(src);
    SET_ZERO(src &= 0xff);
	RET(6);

case 195:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
    PC += 2;

	/* cmp(--src & 0xff));  */

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
	SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
	RET(8);

case 196:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    PC += 2;

    src = YR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
	SET_ZERO(src &= 0xff);
    RET(3);

case 197:
	//p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(LOADEXEC(PC+1));
	PC += 2;

	src = AC - src;
	SET_CARRY(src < 0x100);
	SET_SIGN(src);
	SET_ZERO(src &= 0xff);
	RET(3);

case 198:
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1);
    PC += 2;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
	STORE_ZERO(p1, (src));
	RET(5);

case 199:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1);
    PC += 2;

    /* cmp(--src & 0xff));  */

	src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
	SET_ZERO(AC != src);

    STORE_ZERO(p1, (src));
	RET(5);

case 200:
	PC++;
	YR++;
	SET_SIGN(YR);
	SET_ZERO(YR);
	RET(2);

case 201:
	//p1 = LOADEXEC(PC+1);
	src=LOADEXEC(PC+1);
	PC += 2;

	src = AC - src;
	SET_CARRY(src < 0x100);
	SET_SIGN(src);
	SET_ZERO(src &= 0xff);
	RET(2);

case 202:
	PC++;
	XR--;
	SET_SIGN(XR);
	SET_ZERO(XR);
	RET(2);

case 203:
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
	PC += 2;

    src = (AC & XR) - src;	/* Carry is ignored (CMP) */
    /* Overflow flag may be affected */
    SET_CARRY(src < 0x100);

    src &= 0xff;	/* No decimal mode */
    SET_SIGN(src);
	SET_ZERO(src);

	XR = (src);
	RET(2);

case 204:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

	src = YR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
	SET_ZERO(src &= 0xff);
    RET(4);

case 205:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    PC += 3;

    src = AC - src;
	SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
    RET(4);

case 206:
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2, (src));
	RET(6);

case 207:
	p2 = load_abs_addr;
	src=LOAD(p2);
    PC += 3;

    /* cmp(--src & 0xff));  */

    src = (src - 1) & 0xff;	/* DEC+CMP */
	SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(p2, (src));
	RET(6);

case 208:
/*  BNE,    RELATIVE */
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;
	       /* Branch if  zero flag is clear. */
    if (!IF_ZERO())
    {
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		/* Same page */
			RET(3);
		}
		else
		{
		/* Different page */
			RET(4);
		}
	}
    else
	{
		RET(2);
	}

case 209:
/*   CMP,    INDIRECT_Y */
	//p1 = LOADEXEC(PC+1);
    p2=LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    PC += 2;

	src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);

    if( pagetest(p2,YR))
	{
      /* Over page */
       RET(6);
	}
    else
	{
      /* Same page */
		RET(5);
	}

case 211:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    PC += 2;

    /* cmp(--src & 0xff));  */

    src = (src - 1) & 0xff;	/* DEC+CMP */
	SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(LOAD_ZERO_ADDR(p1) + YR, (src));
	RET(8);

case 213:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    PC += 2;

	src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
    RET(4);

case 214:
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1 + XR);
    PC += 2;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE_ZERO(p1 + XR, (src));
	RET(6);

case 215:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1 + XR);
    PC += 2;

    /* cmp(--src & 0xff));  */

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
	SET_ZERO(AC != src);

    STORE_ZERO(p1 + XR, (src));
	RET(6);

case 216:
    PC++;
    SET_DECIMAL((0));
	RET(2);

case 217:
/*   CMP,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;

    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);

    if( pagetest(p2,YR))
	{
      /* Over page */
	  RET(5);
	}
	else
	{
      /* Same page */
      RET(4);
	}

case 219:
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    PC += 3;

    /* cmp(--src & 0xff));  */

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
    SET_ZERO(AC != src);
    
    STORE(p2 + YR, (src));
	RET(7);

case 221:
/*   CMP,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;
    src = AC - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);

    if( pagetest(p2,XR))
	{
      /* Over page */
       RET(5);
	}
    else
	{
      /* Same page */
	  RET(4);
	}

case 222:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    src = (src - 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);

    STORE(p2 + XR, (src));
	RET(7);

case 223:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    /* cmp(--src & 0xff));  */

    src = (src - 1) & 0xff;	/* DEC+CMP */
    SET_CARRY(AC >= src);
    SET_SIGN(AC - src);
	SET_ZERO(AC != src);
    
    STORE(p2 + XR, (src));
	RET(7);

case 224:
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;

    src = XR - src;
    SET_CARRY(src < 0x100);
    SET_SIGN(src);
    SET_ZERO(src &= 0xff);
    RET(2);

case 225:
	//p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(LOADEXEC(PC+1) + XR));
    temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
	 */

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

	SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
		temp -= 0x60;
	}
	SET_CARRY(temp < 0x100);

	AC = (temp & 0xff);
	RET(6);

case 227:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1 + XR));
    PC += 2;

    /* src = ++src & 0xff; AC = sbc(src); */

    src = ((src + 1) & 0xff);	/* INC+SBC */

   temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
	SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

	if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
		/* src saved */
    STORE(LOAD_ZERO_ADDR(p1 + XR), (src));
	RET(8);

case 228:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    PC += 2;

    src = XR - src;
	SET_CARRY(src < 0x100);
    SET_SIGN(src);
	SET_ZERO(src &= 0xff);
    RET(3);

case 229:
	//p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(LOADEXEC(PC+1));
    temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 2;

	/*
     * SBC is not exact either. It has 6% different results too.
     */


    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
		temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);
    
    AC = (temp & 0xff);
	RET(3);

case 230:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1);
	PC += 2;

	src = (src + 1) & 0xff;
	SET_SIGN(src);
	SET_ZERO(src);

	STORE_ZERO(p1, (src));
	RET(5);

case 231:
	p1 = LOADEXEC(PC+1);
    src=LOAD_ZERO(p1);
    PC += 2;

    /* src = ++src & 0xff; AC = sbc(src); */

    src = ((src + 1) & 0xff);	/* INC+SBC */

   temp = AC - src - (IF_CARRY() ? 0 : 1);

	SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

	AC = temp;
    	/* src saved */
    STORE_ZERO(p1, (src));
	RET(5);

case 232:
	PC++;
	XR++;
	SET_SIGN(XR);
	SET_ZERO(XR);
	RET(2);

case 233:
/*  SBC,    IMMEDIATE */
	p1 = LOADEXEC(PC+1);
    src=p1;
    temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 2;

    /*
	 * SBC is not exact either. It has 6% different results too.
	 */

    if (!IF_DECIMAL()) {
      SET_SIGN(temp);
      SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */
      
      SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));
      SET_CARRY(temp < 0x100);
      AC = (temp & 0xff);

	} else {
      int   bcd1, bcd2;
	  BYTE old_A;
      int C = IF_CARRY() ? 1 : 0;

      old_A = AC;
      
	  bcd1= fromBCD(AC);
      bcd2 = fromBCD(src);
      
      bcd1 = bcd1 - bcd2 - !C;
      
      if (bcd1 < 0) bcd1 = 100 - (-bcd1);
	  AC = toBCD(bcd1);

      SET_CARRY( (old_A < (src + !C)) ? 0 : 1);
      SET_OVERFLOW((old_A ^ AC) & 0x80);      
    }     
    RET(2);

case 235:
	//p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 2;

    /*
	 * SBC is not exact either. It has 6% different results too.
     */

	SET_SIGN(temp);
	SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

	SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

	if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
		temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);
    
    AC = (temp & 0xff);
	RET(2);

case 236:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
	PC += 3;

	src = XR - src;
	SET_CARRY(src < 0x100);
	SET_SIGN(src);
	SET_ZERO(src &= 0xff);
	RET(4);

case 237:
	//p2 = load_abs_addr;
	src=LOAD(load_abs_addr);
    int temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 3;

    /*
	 * SBC is not exact either. It has 6% different results too.
	 */

	SET_SIGN(temp);
	SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

	SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

	if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
		temp -= 6;
	if (temp > 0x99)
		temp -= 0x60;
	}
	SET_CARRY(temp < 0x100);

	AC = (temp & 0xff);
	RET(4);

case 238:
	p2 = load_abs_addr;
	src=LOAD(p2);
	PC += 3;

	src = (src + 1) & 0xff;
	SET_SIGN(src);
	SET_ZERO(src);

	STORE(p2, (src));
	RET(6);

case 239:
	p2 = load_abs_addr;
	src=LOAD(p2);
	PC += 3;

    /* src = ++src & 0xff; AC = sbc(src); */

    src = ((src + 1) & 0xff);	/* INC+SBC */

   temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
		temp -= 0x60;
	}
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
    STORE(p2, (src));
	RET(6);

case 240:
/*   BEQ,    RELATIVE */
//	p1 = LOADEXEC(PC+1);
    src=LOADEXEC(PC+1);
    PC += 2;
	       /* Branch if  zero flag is set. */
    if (IF_ZERO())
    {
		hb=UPPER(PC);
		PC = REL_ADDR(PC, src);
		if( brtest(hb))
		{
		  /* Same page */
		  RET(3);
		}
		else
		{
		  /* Different page */
		  RET(4);
		}
    }
    else
	{
      RET(2);
	}

case 241:
	//p1 = LOADEXEC(PC+1);
    p2=LOAD_ZERO_ADDR(LOADEXEC(PC+1));
	src=LOAD(p2 + YR);
    temp = AC - src - (IF_CARRY() ? 0 : 1);    
	PC += 2;
    
    /*
     * SBC is not exact either. It has 6% different results too.
     */


    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

	if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
	SET_CARRY(temp < 0x100);
    
    AC = (temp & 0xff);

    if( pagetest(p2,YR))
	{
      /* Over page */
      RET(6);
	}
    else
	{
      /* Same page */
	  RET(5);
	}

case 243:
	p1 = LOADEXEC(PC+1);
	src=LOAD(LOAD_ZERO_ADDR(p1) + YR);
    PC += 2;

    /* src = ++src & 0xff; AC = sbc(src); */

    src = ((src + 1) & 0xff);	/* INC+SBC */

   temp = AC - src - (IF_CARRY() ? 0 : 1);

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

	SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);

    AC = temp;
    	/* src saved */
	STORE(LOAD_ZERO_ADDR(p1) + YR, (src));
	RET(8);

case 245:
	//p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(LOADEXEC(PC+1) + XR);
    temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 2;

    /*
     * SBC is not exact either. It has 6% different results too.
     */

    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

	if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
	SET_CARRY(temp < 0x100);
    
    AC = (temp & 0xff);
	RET(4);

case 246:
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1 + XR);
	PC += 2;

	src = (src + 1) & 0xff;
	SET_SIGN(src);
	SET_ZERO(src);

	STORE_ZERO(p1 + XR, (src));
	RET(6);

case 247:
	p1 = LOADEXEC(PC+1);
	src=LOAD_ZERO(p1 + XR);
	PC += 2;

	/* src = ++src & 0xff; AC = sbc(src); */

	src = ((src + 1) & 0xff);	/* INC+SBC */

   temp = AC - src - (IF_CARRY() ? 0 : 1);

	SET_SIGN(temp);
	SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

	SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

	if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
		temp -= 6;
	if (temp > 0x99)
		temp -= 0x60;
	}
	SET_CARRY(temp < 0x100);

	AC = temp;
		/* src saved */
	STORE_ZERO(p1 + XR, (src));
	RET(6);

case 248:
/*  SED,    IMPLIED */
	PC++;
	SET_DECIMAL((1));
	RET(2);

case 249:
/*  SBC,    ABSOLUTE_Y */
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
    temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 3;
    
    /*
     * SBC is not exact either. It has 6% different results too.
	 */

    SET_SIGN(temp);
	SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
		temp -= 0x60;
    }
    SET_CARRY(temp < 0x100);
    
	AC = (temp & 0xff);

	if( pagetest(p2,YR))
	{
	  /* Over page */
      RET(5);
	}
    else
	{
      /* Same page */
      RET(4);
	}

case 251:
	p2 = load_abs_addr;
	src=LOAD(p2 + YR);
	PC += 3;


	/* src = ++src & 0xff; AC = sbc(src); */

	src = ((src + 1) & 0xff);	/* INC+SBC */

	temp = AC - src - (IF_CARRY() ? 0 : 1);

	SET_SIGN(temp);
	SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

	SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

	if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
		temp -= 6;
	if (temp > 0x99)
		temp -= 0x60;
	}
	SET_CARRY(temp < 0x100);

	AC = temp;
    	/* src saved */
    STORE(p2 + YR, (src));
	RET(7);

case 253:
/*   SBC,    ABSOLUTE_X */
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    temp = AC - src - (IF_CARRY() ? 0 : 1);    PC += 3;
 
    /*
     * SBC is not exact either. It has 6% different results too.
     */


    SET_SIGN(temp);
    SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */

    SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));

    if (IF_DECIMAL()) {
	if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
	    temp -= 6;
	if (temp > 0x99)
	    temp -= 0x60;
    }
	SET_CARRY(temp < 0x100);
    
    AC = (temp & 0xff);

    if( pagetest(p2,XR))
	{
      /* Over page */
      RET(5);
	}
	else
	{
      /* Same page */
      RET(4);
	}

case 254:
	p2 = load_abs_addr;
	src=LOAD(p2 + XR);
    PC += 3;

    src = (src + 1) & 0xff;
    SET_SIGN(src);
    SET_ZERO(src);
    
    STORE(p2 + XR, (src));
	RET(7);

case 255:
  p2 = load_abs_addr;
  src=LOAD( p2 + XR);
  PC += 3;
 
  /* src = ++src & 0xff; AC = sbc(src); */
  
  src = ((src + 1) & 0xff);	/* INC+SBC */
  
  temp = AC - src - (IF_CARRY() ? 0 : 1);

  SET_SIGN(temp);
  SET_ZERO(temp & 0xff);	/* Sign and Zero are invalid in decimal mode */
  
  SET_OVERFLOW(((AC ^ temp) & 0x80) && ((AC ^ src) & 0x80));
  
  if (IF_DECIMAL()) {
    if ( ((AC & 0xf) + (IF_CARRY() ? 1 : 0)) < (src & 0xf))
      temp -= 6;
    if (temp > 0x99)
      temp -= 0x60;
  }
  SET_CARRY(temp < 0x100);
  
  AC = temp;
  /* src saved */
  STORE(p2 + XR, (src));
  RET(7);
} /* switch */

//clk+=clkcount;
/*
if (app_data.left == PADDLE) {
	paddle1.val+=clkcount;
	}
	*/
}  /* while (1) */

/*	return b;*/
} /* mainloop */


#ifndef NO_UNDOC_CMDS

void    u_stoshr(unsigned int val, ADDRESS address, BYTE index)
{
    val &= (((address >> 8) + 1) & 0xff);
    if (((address & 0xff) + index) > 0xff)
        address = val;
	STORE((address + index), val);
}
#endif   /* UNDOC */

