/*****************************************************************************

   This file is part of Virtual VCS, the Atari 2600 Emulator
   ===================================================

   Copyright 1996 Alex Hornby. For contributions see the file CREDITS.
   Modified 1996 by Daniel Boris

   This software is distributed under the terms of the GNU General Public
   License. This is free software with ABSOLUTELY NO WARRANTY.

   See the file COPYING for Details.

******************************************************************************/

/* Raster graphics procedures */
#include "sega_scl2.h"

#include <stdio.h>
#include "types.h"
#include "address.h"
#include "vmachine.h"
#include "display.h"
#include "resource.h"
//#include "debug.h"
#include "config.h"
#include "collisio.h"
#include "string.h"

/* Playfield screen position */
Uint32 *pf_pos;
//int ebeamadj;
extern int beamadj;
//unsigned int line_ptr;
//BYTE bkc;

/* Place a playfield pixel */
__inline__ void tv_pfpixel1(void)
{
//	*(pf_pos++)|=PF_MASK;
//	*(pf_pos++)|=PF_MASK;
//	*(pf_pos++)|=PF_MASK;
//	*(pf_pos++)|=PF_MASK;
	*(pf_pos++)|=PF_MASK32;
}

/* Draw playfield register PF0 */
__inline__ void draw_pf0(struct PlayField *pf, int dir) {
	unsigned int pfm; /* playfield mask */
			 /* 1=forward */
	if(dir) 
	{
		for(pfm=0x10;pfm<0x100;pfm<<=1)
		{
			if (pf->pf0 & pfm)
				*(pf_pos)|=PF_MASK32;
			//else
				pf_pos++;
		}
	}
	else
	{
		for(pfm=0x80;pfm >0x08; pfm>>=1)
		{
			if(pf->pf0 & pfm)
				*(pf_pos)|=PF_MASK32;
			//else
				pf_pos++;
		}
	}
}

/* Draw playfield register PF1 */
__inline__ void draw_pf1(struct PlayField *pf, int dir) {
	unsigned int pfm; /* playfield mask */
			 /* 1=forward */
	if(dir) 
	{
		/* do PF1 */
		for(pfm=0x80; pfm>0; pfm>>=1)
		{
			if(pf->pf1 & pfm) 
				*(pf_pos)|=PF_MASK32;
			//else
				pf_pos++;
		}
	} 
	else 
	{
		/* do PF1 */
		for(pfm=0x01;pfm < 0x100;pfm<<=1)
		{
			if(pf->pf1 & pfm)
				*(pf_pos)|=PF_MASK32;
			//else
				pf_pos++;
		}
	}
}

/* Draw playfield register PF2 */
__inline__ void draw_pf2(struct PlayField *pf, int dir) {
	unsigned int pfm; /* playfield mask */
			 /* 1=forward */
	if(dir) 
	{
		/* do PF2 */
		for(pfm=0x01;pfm<0x100; pfm<<=1)
		{
			if (pf->pf2 & pfm)
				*(pf_pos)|=PF_MASK32;
			//else
				pf_pos++;
		}
	}
	else 
	{
		for(pfm=0x80;pfm>0; pfm>>=1)
		{
			if (pf->pf2 & pfm)
				*(pf_pos)|=PF_MASK32;
			//else 
				pf_pos++;
		}
	}
}

/* Update from the playfield display list */
__inline__  void pf_update( int nextx, int *pfc, int pf_max)
{
	for( ;(*pfc < pf_max) && (nextx+3 > pf_change[*pfc].x ); (*pfc)++) 
	{
		use_pfraster_change( &pf, &pf_change[*pfc]);
	}
}

/* Draw the playfield */
__inline__ void draw_playfield(void)
{
//	const int num=0; /* Stick to one playfield */
	unsigned int pfc=0;
	unsigned int pf_max;

	pf_max=pf_change_count;
	pf_pos=(Uint32*)colvect;
	/* First half of playfield */
	//ebeamadj = ebeamx + beamadj;
	pf_update(0, &pfc , pf_max);
	draw_pf0( &pf, 1);
	pf_update(16, &pfc , pf_max);
	draw_pf1( &pf, 1);
	pf_update(48, &pfc , pf_max);
	draw_pf2( &pf, 1);

	pf_update(80, &pfc , pf_max);
	/* Second half of playfield */
	if(pf.ref) {
	draw_pf2( &pf, 0);
	pf_update(112, &pfc , pf_max);
	draw_pf1( &pf, 0);
	pf_update(144, &pfc , pf_max);
	draw_pf0( &pf, 0);
	} else {
	draw_pf0( &pf, 1);
	pf_update(96, &pfc , pf_max);
	draw_pf1( &pf, 1);
	pf_update(128, &pfc , pf_max);
	draw_pf2( &pf, 1);
	}
	/* Use last changes */
	for( ; pfc < pf_max; pfc++)
	use_pfraster_change( &pf, &pf_change[pfc]);

	pf_change_count=0;
}

/* Draws a normal (8 clocks) sized player */
__inline__ void pl_normal( struct Player *p, int x)
{
	/* Set pointer to start of player graphic */
	BYTE *ptr;
	BYTE mask;
	BYTE gr;

	if (p->vdel_flag)
		gr = p->vdel;
	else
		gr = p->grp;

	if (x>=159) x -= 159;

	ptr=colvect+x -1;		/* Point to colvect + x */ /* -1: fix misalignment */
	if(p->reflect) {
		/* Reflected: start with D0 of GRP on left */
		for(mask=0x01;mask>0;mask<<=1) {
			if(gr & mask) {
				*(ptr++)|=p->mask;
			} else
				ptr++;
		}
	}
	else {
		/* Unreflected: start with D7 of GRP on left */
		for(mask=0x80;mask>0;mask>>=1){
			if(gr & mask){
				*(ptr++)|=p->mask;
			} else
				ptr++;
		}
	}
}

/* Draws a double width ( 16 clocks ) player */
void pl_double(struct Player *p, int x)
{
	/* Set pointer to start of player graphic */

	Uint16 *ptr;
	Uint16 mask16;
	BYTE mask;
	BYTE gr;

	if (p->vdel_flag)
		gr = p->vdel;
	else
		gr = p->grp;

	if (x>=159) x -= 159;
	mask16 = p->mask|(p->mask<<8); 
	ptr=(Uint16 *)colvect+(x) -1;
	if(p->reflect) {
	for(mask=0x01;mask>0;mask<<=1){
		if(gr & mask){
		//*(ptr++)|=p->mask;	  
		//*(ptr++)|=p->mask;
		*(ptr++)|=mask16;
		} else
		//ptr+=2;
		ptr++;
	}
	}
	else {
	for(mask=0x80;mask>0;mask>>=1){
		if(gr & mask){
		//*(ptr++)|=p->mask;	  
		//*(ptr++)|=p->mask;
		*(ptr++)|=mask16;
		} else
		//ptr+=2;
		ptr++;
	}
	}
}

/* Draws a quad sized ( 32 clocks) player */
void pl_quad(struct Player *p, int x)
{
	/* Set pointer to start of player graphic */
	Uint32 *ptr;
	Uint32 mask32;
	BYTE mask;
	BYTE gr;

	if (p->vdel_flag)
		gr = p->vdel;
	else
		gr = p->grp;

	if (x>=159) x -= 159;
	mask32 = p->mask | (p->mask<<8) | (p->mask<<16) | (p->mask<<24);
	ptr=(Uint32 *)colvect+x -1;
	if(p->reflect) {
	for(mask=0x01;mask>0;mask<<=1){
		if(gr & mask){
		//*(ptr++)|=p->mask;	  
		//*(ptr++)|=p->mask;
		//*(ptr++)|=p->mask;	  
		//*(ptr++)|=p->mask;
		*(ptr++)|=mask32;
		} else
		//ptr+=4;
		ptr++;
	}
	}
	else {
	for(mask=0x80;mask>0;mask>>=1){
		if(gr & mask){
		//*(ptr++)|=p->mask;	  
		//*(ptr++)|=p->mask;
		//*(ptr++)|=p->mask;	  
		//*(ptr++)|=p->mask;
		*(ptr++)|=mask32;
		} else
		//ptr+=4;
		ptr++;
	}
	}
}



/* Consume the player display list */
__inline__ void pl_update( int num, int nextx, int plc, int pl_max)
{
	for( ;(plc < pl_max) && (nextx > pl_change[num][plc].x); plc++) {
	use_plraster_change( &pl[num], &pl_change[num][plc]);
	}
}

/* Draw a player */
__inline__ void pl_draw1(int line,int num)
{
	unsigned int plc=0;
	unsigned int pl_max=pl_change_count[num];
	int nextx;

	pl_update(num, pl[num].x, plc , pl_max);

	if(pl[num].x>=0 ) {

	/*if(pl_max > plc)
		use_plraster_change( &pl[num], &pl_change[num][plc++]);*/
	switch(pl[num].nusize){
	case 0:
		/* One copy */
		pl_normal(&pl[num], pl[num].x);
		break;
	case 1:
		/* Two copies close */
		pl_normal(&pl[num], pl[num].x);
		//nextx=pl[num].x + 8 +8;
		nextx=pl[num].x + 16;
		pl_update(num, nextx, plc , pl_max);
		pl_normal(&pl[num], nextx);
		break;
	case 2:
		/* Two copies medium */
		pl_normal(&pl[num], pl[num].x);
		//nextx=pl[num].x + 8 +24;
		nextx=pl[num].x + 32;
		pl_update(num, nextx, plc , pl_max);
		pl_normal(&pl[num], nextx);
		break;
	case 3:
		/* Three copies close */
		/* Pacman score line */
		pl_normal(&pl[num], pl[num].x);

		nextx=pl[num].x + 16;
		pl_update(num, nextx, plc , pl_max);
		pl_normal(&pl[num], nextx);

		nextx=pl[num].x + 32;
		pl_update(num, nextx, plc , pl_max);

		pl_normal(&pl[num], nextx);
		break;
	case 4:
		/* Two copies wide */
		pl_normal(&pl[num], pl[num].x);
		//nextx=pl[num].x + 8 +56;
		nextx=pl[num].x + 64;
		pl_update(num, nextx, plc , pl_max);
		pl_normal(&pl[num], nextx);
		break;
	case 5:
		/* Double sized player */
		pl_double(&pl[num], pl[num].x);
		break;
	case 6:
		/* Three copies medium */
		pl_normal(&pl[num], pl[num].x);
		//nextx=pl[num].x + 8 +24;
		nextx=pl[num].x + 32;
		pl_update(num, nextx, plc , pl_max);
		pl_normal(&pl[num], nextx);
		//nextx=pl[num].x + 8 +56;
		nextx=pl[num].x + 64;
		pl_update(num, nextx, plc , pl_max);
		pl_normal(&pl[num], nextx);
		break;
	case 7:
		/* Quad sized player */
		pl_quad(&pl[num], pl[num].x);
		break;
	}
	}

	/* Use last changes */
	for( ; plc < pl_max; plc++)
	use_plraster_change( &pl[num], &pl_change[num][plc]);

	pl_change_count[num]=0;
}


/* Draw the ball */
static void draw_ball(int line){
	//int i;
	BYTE e;

	if (ml[2].vdel_flag)
		e = ml[2].vdel;
	else
		e = ml[2].enabled;


	if(e && ml[2].x >= 0 ){
	BYTE *blptr;
		blptr=colvect+(ml[2].x) -1;		/* Fix for misalignment */
		switch(tiaWrite[CTRLPF] >> 4) {
		case 3:
			/* Eight clocks */
			//for(i=0;i<8;i++)
			//*(blptr++)|=BL_MASK;
			//break;
			*((Uint32*)blptr++)|=BL_MASK32;
			*((Uint32*)blptr++)|=BL_MASK32;
			break;
		case 2:
			/* Four clocks */
			//for(i=0;i<4;i++)
			//*(blptr++)|=BL_MASK;
			//*(blptr++)|=BL_MASK;
			*((Uint32*)blptr++)|=BL_MASK32;
			break;
		case 1:
			/* Two clocks */
			//for(i=0;i<2;i++)
			//*(blptr++)|=BL_MASK;
			*((Uint16*)blptr++)|=BL_MASK16;
			break;
		case 0:
			/* One clock */
			*(blptr++)|=BL_MASK;
			break;
		}
	}
}

__inline__  void do_missile(int num, BYTE *misptr)
{
	//int i;
//#ifdef xxx

	switch( ml[num].width) {
		case 3:
			/* Eight clocks */
			//for(i=0;i<8;i++)
			//*(misptr++)|=ml[num].mask;
			//break;
			//*(misptr++)|=ml[num].mask;
			//*(misptr++)|=ml[num].mask;
			//*(misptr++)|=ml[num].mask;
			//*(misptr++)|=ml[num].mask;
			*((Uint32*)misptr++)|=(ml[num].mask|(ml[num].mask<<8)|(ml[num].mask<<16)|(ml[num].mask<<24));
			*((Uint32*)misptr++)|=(ml[num].mask|(ml[num].mask<<8)|(ml[num].mask<<16)|(ml[num].mask<<24));
			break;
		case 2:
			/* four clocks */
			//for(i=0;i<4;i++)
			//*(misptr++)|=ml[num].mask;
			//*(misptr++)|=ml[num].mask;
			//*(misptr++)|=ml[num].mask;
			//*(misptr++)|=ml[num].mask;
			*((Uint32*)misptr++)|=(ml[num].mask|(ml[num].mask<<8)|(ml[num].mask<<16)|(ml[num].mask<<24));
			break;
		case 1:
			/* two clocks */
			//for(i=0;i<2;i++)
			//*(misptr++)|=ml[num].mask;
			*((Uint16*)misptr++)|=(ml[num].mask|(ml[num].mask<<8));
			break;
		case 0:
			/* one clock */
			*(misptr++)|=ml[num].mask;
			break;
	}   /* switch */
//#endif
}

/* Draw the missile */
static void draw_missile(int line, int num){
//	int i;
	BYTE *misptr;

	if(ml[num].enabled && ml[num].x>=0){
		misptr=colvect+(ml[num].x) -1;

		switch(pl[num].nusize){
		case 0:
		//	misptr=colvect+(ml[num].x) -1;
		//	do_missile(num,misptr);
			break;
		case 1:
			//misptr=colvect+(ml[num].x) -1;
			do_missile(num,misptr);
			misptr=misptr+16;
			//do_missile(num,misptr);
			break;
		case 2:
			//misptr=colvect+(ml[num].x) -1;
			do_missile(num,misptr);
			misptr=misptr+32;
			//do_missile(num,misptr);
			break;
		case 3:
			//misptr=colvect+(ml[num].x) -1;
			do_missile(num,misptr);
			misptr=misptr+16;
			do_missile(num,misptr);
			misptr=misptr+16;
			//do_missile(num,misptr);
			break;
		case 4:
			//misptr=colvect+(ml[num].x) -1;
			do_missile(num,misptr);
			misptr=misptr+64;
			//do_missile(num,misptr);
			break;
		case 5:
			//misptr=colvect+(ml[num].x) -1;
			//do_missile(num,misptr);
			break;
		case 6:
			//misptr=colvect+(ml[num].x) -1;
			do_missile(num,misptr);
			misptr=misptr+32;
			do_missile(num,misptr);
			misptr=misptr+32;
			//do_missile(num,misptr);
			break;
		case 7:
			//misptr=colvect+(ml[num].x) -1;
			//do_missile(num,misptr);
			break;

		}
		do_missile(num,misptr);

	} /* If */
}


/* Draw one tv raster line */
__inline__ void tv_raster1( int line )
{
/*	line_ptr=line*vwidth;*/
/*	line_ptr=line;*/
//	line_ptr=239-line;
//	bkc=tiaWrite[COLUBK];
	//ebeamadj = ebeamx + beamadj;

	if(tiaWrite[CTRLPF] & 0x02) 
	{
		/* Alterate priority */
		draw_missile(line,1);
		pl_draw1(line,1);

		/* Do the missiles */
		draw_missile(line,0);
		pl_draw1(line,0);

		/* Do the ball */
		draw_ball(line);

		/* Do the playfield */
		draw_playfield();
	} 
	else 
	{
		/* Normal Priority */
		draw_playfield();		  //prend un peu de temps

		/* Do the ball */
		 draw_ball(line);	   //prend zero

		/* Do the player 1 graphics */
		draw_missile(line,1);	   //prend zero
		pl_draw1(line,1);		 //prend un peu de temps

		/* Do the player 0 graphics */
		draw_missile(line,0);
		pl_draw1(line,0);		//prend un peu de temps
	}
}

/* Reset the collision vector */
/*void reset_vector(void)
{
	int i;
	//for(i=0;i<160;i++) colvect[i]=0;
	memset(colvect,0x00,160);
}			 */
/* draw the collision vector */
 void draw_vector1(unsigned int line)
{
	unsigned int i;
	unsigned int uct=0;
	//BYTE pad=0;
	//unsigned int tv_ptr;
	//BYTE b;

	//tv_ptr=line+1;//240-line_ptr;
	/* Use starting changes */
	while(uct < unified_count && unified[uct].x<0)
	use_unified_change(&unified[uct++]);

	Uint8   	*VramWorkP;

	VramWorkP = (Uint8 *)SCL_VDP2_VRAM_A1+((line+10)<<9);

	for(i=0; i< TV_WIDTH; i++) 
	{
  
		if(uct < unified_count && unified[uct].x==i)
			use_unified_change(&unified[uct++]);

		if (coltable[colvect[i]]) 
		{
			set_collisions(colvect[i]);
		}

		if(tiaWrite[CTRLPF] & 0x04) 
		{
			//if(colvect[i] & (BL_MASK | PF_MASK)) 
			if(colvect[i] & COL13)
			{
				if (tiaWrite[CTRLPF] & 0x02) 
				{
					
					if (i > 79) *(VramWorkP++)=pl[1].col;
					else	 *(VramWorkP++)=pl[0].col;
				}
				else *(VramWorkP++)=pf.col;
			}
			//else if(colvect[i] & (PL0_MASK | ML0_MASK))
			else if(colvect[i] & COL2) *(VramWorkP++)=pl[0].col;
			//else if(colvect[i] & (PL1_MASK | ML1_MASK))
			else if(colvect[i] & COL4) 	*(VramWorkP++)=pl[1].col;
			else *(VramWorkP++)=tiaWrite[COLUBK];
		}
		else 
		{
			/* Normal Priority */
			//if(colvect[i] & (PL0_MASK | ML0_MASK))
			if(colvect[i] & COL2) *(VramWorkP++)=pl[0].col;
			//else if(colvect[i] & (PL1_MASK | ML1_MASK))
			else if(colvect[i] & COL4) *(VramWorkP++)=pl[1].col;
			//else if(colvect[i] & (BL_MASK | PF_MASK)) 
			else if(colvect[i] & COL13) 
			{
				if (tiaWrite[CTRLPF] & 0x02) 
				{
					if (i > 79) *(VramWorkP++)=pl[1].col;
					else *(VramWorkP++)=pl[0].col;
				}
				else *(VramWorkP++)=pf.col;
			}
			else *(VramWorkP++)=tiaWrite[COLUBK];
		}
		//*(VramWorkP++)=pad;
	}
	
	while(uct < unified_count)
		use_unified_change(&unified[uct++]);
	
	unified_count=0;
}

/* Used for when running in frame skipping mode */
static void update_registers(void)
{
	unsigned int i,num;

	/* Playfield */
	for(i=0 ; i < pf_change_count; i++)
	use_pfraster_change( &pf, &pf_change[i]);
	pf_change_count=0;

	/* Player graphics */
	for(num=0; num < 2; num ++) 
	{
		for(i=0 ; i < pl_change_count[num]; i++)
			use_plraster_change( &pl[num], &pl_change[num][i]);
		pl_change_count[num]=0;
	}

	/* Unified */
	for(i=0; i < unified_count; i++)
	use_unified_change(&unified[i]);
	unified_count=0;
}



/* Main raster function, will have switches for different magsteps */
__inline__ void tv_raster( int line ){
	
	if(tv_counter % app_data.rr != 0) {
		update_registers();
		return;
	}
	
	//reset_vector();
	Uint32 *vbt=(Uint32*)colvect;

	unsigned int i;
	for(i=0;i<40;i++) vbt[i]=0;
//	memset(vbt,0x0000,40);


	tv_raster1(line);
	if (line < 200) {
		draw_vector1(line);
	}
//#endif
//update_registers();
}
